% lda computes an approximate LDA geodesic segment by selecting the optimal 
% segment between randomly sampled data points
%
% Defined in the paper 'Tree-space statistics and approximations for 
% large-scale analysis of anatomical trees (Aasa Feragen, Megan Owen, Jens 
% Petersen, Mathilde M. W. Wille, Laura H. Thomsen, Asger Dirksen, and 
% Marleen de Bruijne), Information Processing in Medical Imaging 2013, 
% Lecture Notes in Computer Science 7917'.
%
% [start_index, end_index, classification_accuracy] = lda(num_iter, epsilon, tree_file, classification_vector, training_test)
%
% Input variables:
%  num_iter                number of sampled endpoint pairs
%  epsilon                 parameter determining the convergence of
%                          projection
%  treefile                the path of a file containing N data
%                          trees in Newick format, one per line
%  classification_vector   an 1 x N binary vector (0/1) indicating the 
%                          classes of the N trees
%  training_test           a 1 x N binary vector (0/1) indicating the
%                          training (1) or test (0) set membership of the 
%                          N trees
%
% Output variables:
%  start_tree              starting tree of optimal geodesic segment
%  end_tree                endpoint tree of optimal geodesic segment
%  classification_accuracy accuracy of  classification by projecting onto
%                          the optimal segment
%
% Copyright (C) 2013 Aasa Feragen
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%
% Author: Aasa Feragen <aasa at diku dot dk>
% 
% 2013-06-16 Aasa Feragen <aasa at diku dot dk>
% * Initial revision

function [start_tree, end_tree, classification_accuracy] = lda(num_iter, epsilon, tree_file, classification_vector, training_test)

    if nargin < 2
        epsilon = 0.0005;
    end
    
    if nargin < 3
        tree_file = 'airways_dataset.txt';
    end
    
    %----------------------------------------------------------------------
    
    mkdir intermediate_files
    % Make a directory for intermediate files
    
    %----------------------------------------------------------------------
    % Make txt files for the test set and the two classes in the training
    % set.
    
    training_A = find(training_test.*classification_vector == 1);    
    training_B = find(training_test.*(1-classification_vector) == 1);
    test_A = find((1-training_test).*classification_vector == 1);
    test_B = find((1-training_test).*(1-classification_vector) == 1);
    test = find(training_test == 0);
    classification_vector_test = classification_vector(test);
    
    fid = fopen(tree_file);
    s = textscan(fid, '%s', 'delimiter', '\n', 'BufSize', 100000);
    s = s{1};
    fclose(fid);
    
    fid = fopen('intermediate_files/training_A.txt', 'w');    
    for i = 1 : numel(training_A)    
        fprintf(fid, [s{training_A(i)}, '\n']);
    end
    fclose(fid);
    
    fid = fopen('intermediate_files/training_B.txt', 'w');    
    for i = 1 : numel(training_B)    
        fprintf(fid, [s{training_B(i)}, '\n']);
    end
    fclose(fid);
    
    fid = fopen('intermediate_files/test.txt', 'w');    
    for i = 1 : numel(test)    
        fprintf(fid, [s{test(i)}, '\n']);
    end
    fclose(fid);
    
    %----------------------------------------------------------------------
    %
    % Run LDA
        
    system(['java -jar LDA.jar -a random -e ', num2str(epsilon), ' -i ', num2str(num_iter), ' -o intermediate_files/lda -f intermediate_files/training_A.txt -g intermediate_files/training_B.txt ' tree_file])

    %----------------------------------------------------------------------
    %
    % Evaluate results
    
    a = dir(fullfile('intermediate_files', 'lda*'));
    
    for i = 1 : numel(a)    
        display(['Evaluating LDA ', num2str(i)]);
        [~, name, ~] = fileparts(a(i).name);
        fid = fopen(fullfile('intermediate_files', name));
        s = textscan(fid, '%s', 'delimiter', '\n', 'BufSize', 100000);
        s = s{1};
        lda(i).tree1 = s{1};
        lda(i).tree2 = s{2};
        scorestring = s{3};
        lda(i).score = str2num(scorestring(8:end));
        mean1 = s{4};
        mean2 = s{5};
        lda(i).mean1 = str2num(mean1(9:end));
        lda(i).mean2 = str2num(mean2(9:end));
        clear s scorestring mean1 mean2
        fclose(fid);
    end
    
    % Results stored in struct lda
    
    scores = [lda(:).score];
    maximum = max(scores);
    maximum_index = find(scores == maximum);
    maximum_index = maximum_index(1);
    % Choose the first option if there are two minimizers
    
    start_tree = lda(maximum_index).tree1;
    end_tree = lda(maximum_index).tree2;
    mean_A = lda(maximum_index).mean1;
    mean_B = lda(maximum_index).mean2;
    
    fid = fopen('intermediate_files/endpoints.txt', 'w');
    fprintf(fid, [start_tree, '\n']);
    fprintf(fid, [end_tree, '\n']);
    fclose(fid);
    
    %----------------------------------------------------------------------
    %
    % Compute classification accuracy
    
    system(['java -jar LDA.jar -a classify -e ', num2str(epsilon), ' -f intermediate_files/endpoints.txt  -o intermediate_files/classification.txt -m ', num2str(mean_A), ',', num2str(mean_B), ' intermediate_files/test.txt']);
    
    fid = fopen('intermediate_files/classification.txt');
    s = textscan(fid, '%s', 'delimiter', '\n', 'BufSize', 100000);
    s = s{1};
    fclose(fid);
    
    for i = 1 : numel(s)
        classification(i) = str2num(s{i});
    end
    classification_success = (classification == classification_vector_test);
    classification_accuracy = sum(classification_success)/numel(classification_success);
end