% pca computes an approximate principal component geodesic segment by
% selecting the optimal segment between randomly sampled data points
%
% Defined in the paper 'Tree-space statistics and approximations for 
% large-scale analysis of anatomical trees (Aasa Feragen, Megan Owen, Jens 
% Petersen, Mathilde M. W. Wille, Laura H. Thomsen, Asger Dirksen, and 
% Marleen de Bruijne), Information Processing in Medical Imaging 2013, 
% Lecture Notes in Computer Science 7917'.
%
% [start_index, end_index] = pca(num_iter, precision, tree_file)
%
% Input variables:
%  num_iter                number of sampled endpoint pairs
%  precision               parameter determining the convergence of
%                          projection
%  treefile                the string name of a file containing all data
%                          trees in Newick format, one per line
%
% Output variables:
%  start_index             index of starting point of optimal geodesic
%                          segment
%  end_index               index of endpoint of optimal geodesic segment
%
% Copyright (C) 2013 Aasa Feragen
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
%
% Author: Aasa Feragen <aasa at diku dot dk>
% 
% 2013-06-16 Aasa Feragen <aasa at diku dot dk>
% * Initial revision

function [start_index, end_index] = pca(num_iter, precision, tree_file)

    if nargin < 2
        precision = 0.0005;
    end
    
    if nargin < 3
        tree_file = 'airways_dataset.txt';
    end
    
    mkdir intermediate_files
    % Make a directory for intermediate files
    system(['java -jar PCA.jar -i ', num2str(num_iter), ' -a random -r -e ', num2str(precision), ' -o intermediate_files/pca ', tree_file]);
    
    a = dir(fullfile('intermediate_files', 'pca*'));
    for i = 1 : numel(a)        
        [~, name, ~] = fileparts(a(i).name);
        fid = fopen(fullfile('intermediate_files', name));
        s = textscan(fid, '%f\t%f\t%f\t%f\t%f', 'delimiter', '\n', 'BufSize', 100000, 'headerLines', 1);
        pca(i).index1 = s{1};
        pca(i).index2 = s{2};
        pca(i).num_orthants = s{3};
        pca(i).geo_length = s{4};
        pca(i).score = s{5};
        clear s
        fclose(fid);
    end
    scores = [pca(:).score];
    minimum = min(scores);
    minimum_index = find(scores == minimum);
    minimum_index = minimum_index(1);
    % Choose the first option if there are two minimizers
    
    start_index = pca(minimum_index).index1 + 1;
    end_index = pca(minimum_index).index2 + 1;
end